----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Joachim Schmidt <joachim.schmidt@hesge.ch>
--
-- Module Name: tb_scalp_rr_scheduler.vhd
-- Target Device: SCALP xc7z015clg485-2
-- Tool version: 2019.1
-- Description: Testbench for scalp_rr_scheduler.
--
-- Last update: 2020-01-17
--
---------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
-- use ieee.math_real.uniform;
-- use ieee.math_real.floor;

--use ieee.std_logic_unsigned.all;
--use ieee.std_logic_arith.all;
--use ieee.std_logic_misc.all;

--library UNISIM;
--use UNISIM.VCOMPONENTS.all;

library work;
use work.scalp_misc.all;
-- use work.scalp_utility.all;

entity tb_scalp_scheduler is

end entity tb_scalp_scheduler;

architecture testbench of tb_scalp_scheduler is

    constant C_SCALP_STRATEGY   : string   := "RR";
    constant C_SCALP_PORTS      : positive := 6;
    constant C_SCALP_OUTPUT_REG : boolean  := false;
    constant C_CLK_PERIOD       : time     := 10 ns;

    component scalp_scheduler is
        generic (
            C_SCALP_STRATEGY   : string;
            C_SCALP_PORTS      : positive;
            C_SCALP_OUTPUT_REG : boolean);
        port (
            SysClkxCI        : in  std_ulogic;
            SysRstxRNAI      : in  std_ulogic;
            ArbitratexSI     : in  std_ulogic;
            RequestVectorxDI : in  std_ulogic_vector((C_SCALP_PORTS - 1) downto 0);
            ArbitratedxSO    : out std_ulogic;
            GrantVectorxDO   : out std_ulogic_vector((C_SCALP_PORTS - 1) downto 0);
            GrantIndexxDO    : out std_ulogic_vector((scalp_log2ceilnz(C_SCALP_PORTS) - 1) downto 0));
    end component scalp_scheduler;

    -- Entity signals
    signal SysClkxC        : std_ulogic                                                        := '0';
    signal SysRstxRNA      : std_ulogic                                                        := '0';
    signal ArbitratexS     : std_ulogic                                                        := '0';
    signal RequestVectorxD : std_ulogic_vector((C_SCALP_PORTS - 1) downto 0)                   := (others => '0');
    signal ArbitratedxS    : std_ulogic                                                        := '0';
    signal GrantVectorxD   : std_ulogic_vector((C_SCALP_PORTS - 1) downto 0)                   := (others => '0');
    signal GrantIndexxD    : std_ulogic_vector((scalp_log2ceilnz(C_SCALP_PORTS) - 1) downto 0) := (others => '0');
    -- Simulation signals
    signal EndSimxS        : boolean                                                           := false;

begin  -- architecture testbench

    AsnycStatementsxB : block is
    begin  -- block AsnycStatementsxB
        SysClkxAS : SysClkxC   <= not SysClkxC after (C_CLK_PERIOD / 2);
        SysRstxAS : SysRstxRNA <= '1'          after (C_CLK_PERIOD * 10);
    end block AsnycStatementsxB;

    ScalpSchedulerxI : entity work.scalp_scheduler
        generic map (
            C_SCALP_STRATEGY   => C_SCALP_STRATEGY,
            C_SCALP_PORTS      => C_SCALP_PORTS,
            C_SCALP_OUTPUT_REG => C_SCALP_OUTPUT_REG)
        port map (
            SysClkxCI        => SysClkxC,
            SysRstxRNAI      => SysRstxRNA,
            ArbitratexSI     => ArbitratexS,
            RequestVectorxDI => RequestVectorxD,
            ArbitratedxSO    => ArbitratedxS,
            GrantVectorxDO   => GrantVectorxD,
            GrantIndexxDO    => GrantIndexxD);

    EndSimxP : process is
    begin  -- process EndSimxP
        if EndSimxS = true then
            assert false report "End of simulation!" severity failure;
        end if;

        wait until rising_edge(SysClkxC);
    end process EndSimxP;

    ScalpSchedulerRequestxP : process is
    begin  -- process ScalpSchedulerRequestxP
        ArbitratexS     <= '0';
        RequestVectorxD <= (others => '0');

        wait until (SysRstxRNA = '1');
        wait until rising_edge(SysClkxC);
        wait until rising_edge(SysClkxC);

        for j in 0 to 3 loop
            ArbitratexS     <= '1';
            RequestVectorxD <= "100001";
            for i in 0 to 8 loop
                wait until rising_edge(SysClkxC);
            end loop;  -- i
            RequestVectorxD <= "101101";
            for i in 0 to 3 loop
                wait until rising_edge(SysClkxC);
            end loop;  -- i
            RequestVectorxD <= "000000";
            for i in 0 to 2 loop
                wait until rising_edge(SysClkxC);
            end loop;  -- i
            RequestVectorxD <= "111111";
            for i in 0 to 9 loop
                wait until rising_edge(SysClkxC);
            end loop;  -- i

            ArbitratexS     <= '0';
            RequestVectorxD <= "000000";
            for i in 0 to 3 loop
                wait until rising_edge(SysClkxC);
            end loop;  -- i
        end loop;  -- j

        for i in 0 to 15 loop
                wait until rising_edge(SysClkxC);
            end loop;  -- i

        for j in 0 to 1 loop
            ArbitratexS <= '1';
            RequestVectorxD <= "111111";
            for i in 0 to 9 loop
                wait until rising_edge(SysClkxC);
            end loop;  -- i
            ArbitratexS <= '0';
            for i in 0 to 1 loop
                wait until rising_edge(SysClkxC);
            end loop;  -- i
            ArbitratexS <= '1';
            for i in 0 to 9 loop
                wait until rising_edge(SysClkxC);
            end loop;  -- i
        end loop;  -- j

        EndSimxS <= true;
    end process ScalpSchedulerRequestxP;

end architecture testbench;
